"""
Used to hold all constants related to AIDE. These should only be program specific constants and NOT
expert input constants (physical quantities). Those go in expert_inputs.
"""

DEFAULT_SIGNIFICANT_FIGURES = 4
